@@ -17,16 +17,16 @@ ITEM_VTOLERANCY = 5
 
 STANDALONE_FIELDS = [
 	'CIP',
-	'Data obtenció mostra',
-	'Data recepció mostra',
+	'Data obtencio mostra',
+	'Data recepcio mostra',
 	'Edat',
-	'Localització',
+	'Localitzacio',
 	'Metge',
 	'N Laboratori',
 	'NHC',
 	'Observacions',
 	'Pacient',
-	'Procedència',
+	'Procedencia',
 	'Servei',
 	'Sexe',
 	'Unitat de tractament',
@@ -37,108 +37,121 @@ FIELD_MAPPING = {
 	# Standalone Fields
 
 	'CIP': 'Pacient/CIP',
-	'Data obtenció mostra': 'Peticio/Data',
+	'Data obtencio mostra': 'Peticio/Data',
 	'Edat': 'Pacient/Edat',
-	'Localització': 'Pacient/Ubicacio',
+	'Localitzacio': 'Pacient/Ubicacio',
 	'Metge': 'Peticio/Solicitant',
 	'N Laboratori': 'Peticio/ID',
 	'NHC': 'Pacient/NHC',
 	'Observacions': 'Peticio/Observacions',
 	'Pacient': 'Pacient/Nom',
-	'Procedència': 'Peticio/Hospital',
+	'Procedencia': 'Peticio/Hospital',
 	'Servei': 'Pacient/Servei',
 	'Sexe': 'Pacient/Sexe',
 	'Unitat de tractament': 'Pacient/Unitat',
 
 	# Regular fields
 
-	'Ers(San)-Hemoglobina;c.massa (CHCM)': 'Sang/CHCM',
-	'Ers(San)-Hemoglobina;massa entítica (HCM)': 'Sang/HCM',
-	'Ers(San)-Volum eritrocític;amplada de la distribució rel.': 'Sang/ADE',
-	'Gas(vSan)-Diòxid de carboni;pr.parc.': 'Sang(v)/pCO2',
+	'Ers(San)-Hemoglobina;c.massa(CHCM)': 'Sang/CHCM',
+	'Ers(San)-Hemoglobina;massa entitica(HCM)': 'Sang/HCM',
+	'Ers(San)-Reticulocits;fr.nom.': 'Sang/%Reticulocits',
+	'Ers(San)-Volum eritrocitic;amplada de la distribucio rel.': 'Sang/ADE',
+	'Gas(vSan)-Dioxid de carboni;pr.parc.': 'Sang(v)/pCO2',
 	'Gas(vSan)-Oxigen;pr.parc.': 'Sang(v)/pO2',
 	'Hb(San)-Hemoglobina A1c;fr.subst.(IFCC)': 'Sang/HbA1c',
 	'Hb(San)-Hemoglobina A1c;fr.subst.(expressat en %)': 'Sang/%HbA1c',
 	'Hb(vSan)-Oxigen;fr.sat.': 'Sang(v)/%SatHb',
-	'Lks(San)-Basòfils;fr.nom.': 'Sang/%Basofils',
-	'Lks(San)-Eosinòfils;fr.nom.': 'Sang/%Eosinofils',
-	'Lks(San)-Limfòcits;fr.nom.': 'Sang/%Limfocits',
-	'Lks(San)-Monòcits;fr.nom.': 'Sang/%Monocits',
-	'Lks(San)-Neutròfils(segmentats);fr.nom.': 'Sang/%Neutrofils',
+	'Lks(San)-Basofils;fr.nom.': 'Sang/%Basofils',
+	'Lks(San)-Eosinofils;fr.nom.': 'Sang/%Eosinofils',
+	'Lks(San)-Limfocits;fr.nom.': 'Sang/%Limfocits',
+	'Lks(San)-Metamielotics;fr.nom': 'Sang/%Metamielocits',
+	'Lks(San)-Monocits;fr.nom.': 'Sang/%Monocits',
+	'Lks(San)-Neutrofils(segmentats);fr.nom.': 'Sang/%Neutrofils',
 	'Pac(vSan)-Plasma;pH': 'Sang(v)/pH',
 	'Pla-Alanina-aminotransferasa;c.cat.': 'Serum/ALT',
-	'Pla-Anticoagulant lúpic;c.arb.(negatiu;dubtós;positiu)': 'Plasma/Anticoagulant-Lupic',
+	'Pla-Anticoagulant lupic;c.arb.(negatiu;dubtos;positiu)': 'Plasma/Anticoagulant-Lupic',
+	'Pla-Aspartat-aminotransferasa;c.cat.': 'Serum/AST',
 	'Pla-Bilirubina;c.subst.': 'Serum/Bilirrubina',
 	'Pla-Calci(II);c.subst.': 'Serum/Calci',
-	'Pla-Clorur;c.subst': 'Serum/Clorur',
-	'Pla-Coagulació induida per factor tissular;INR(temps': 'Plasma/INR',
-	'Pla-Coagulació induida per factor tissular;temps rel.(temps': 'Plasma/TP',
-	'Pla-Coagulació induida per una superfície;temps rel.(TTPA)': 'Plasma/TTPA',
+	'Pla-Clorur;c.subst.': 'Serum/Clorur',
+	'Pla-Coagulacio induida per factor tissular;INR(temps de': 'Plasma/INR',
+	'Pla-Coagulacio induida per factor tissular;INR(temps': 'Plasma/INR',
+	'Pla-Coagulacio induida per factor tissular;temps rel.(temps': 'Plasma/TP',
+	'Pla-Coagulacio induida per una superficie;temps rel.(TTPA)': 'Plasma/TTPA',
 	'Pla-Creatina-cinasa;c.cat.': 'Serum/CK',
 	'Pla-Creatinini;c.subst.': 'Serum/Creatinina',
 	'Pla-Fibrinogen;c.massa(coagul.;Clauss)': 'Plasma/Fibrinogen',
+	'Pla-Fibrinogen(coag);c.massa': 'Plasma/Fibrinogen',
+	'Pla-Fibrinogen;c.massa(coagul.;derivat)': 'Plasma/Fibrinogen',
 	'Pla-Glucosa;c.subst.': 'Sang/Glucosa',
-	'Pla-Ió potassi;c.subst.': 'Serum/Potassi',
-	'Pla-Ió sodi;c.subst.': 'Serum/Sodi',
+	'Pla-Io calci;c.subst.(pH 7.4)': 'Serum/Calci(ionitzat)',
+	'Pla-Io potassi;c.subst.': 'Serum/Potassi',
+	'Pla-Io sodi;c.subst.': 'Serum/Sodi',
+	'Pla-Lactat;c.subst.': 'Serum/Lactat',
 	'Pla-Proteina C reactiva;c.massa(CRM 470)': 'Serum/PCR',
 	'Pla-Proteina;c.massa': 'Serum/Proteines',
 	'Pla-Troponina T;c.massa': 'Serum/TnT',
 	'Pla-Urea;c.subst.': 'Sang/Urea',
 	'Pla-alfa-Amilasa;c.cat.': 'Serum/Amilasa',
-	'Pla—Aspartat-aminotransferasa;c.cat.': 'Serum/AST',
-	'Ren-Filtrat glomerular;cabal vol.(equació CKD-EPI)': 'Renal/Filtrat(CKD-EPI)',
-	'San-Basòfils;c.nom.': 'Sang/Basofils',
-	'San-Eosinòfils;c.nom.': 'Sang/Eosinofils',
-	'San-Eritrosedimentació;long.': 'Sang/VSG',
-	'San-Eritròcits;c.nom.': 'Sang/Eritrocits',
-	'San-Eritròcits;fr.vol.(hematòcrit)': 'Sang/Hematocrit',
-	'San-Eritròcits;vol.entític(VCM)': 'Sang/VCM',
+	'Ren-Filtrat glomerular;cabal vol.(equacio CKD-EPI)': 'Renal/Filtrat(CKD-EPI)',
+	'San-Basofils;c.nom.': 'Sang/Basofils',
+	'San-Eosinofils;c.nom.': 'Sang/Eosinofils',
+	'San-Eritrosedimentacio;long.': 'Sang/VSG',
+	'San-Eritrocits;c.nom.': 'Sang/Eritrocits',
+	'San-Eritrocits;fr.vol.(hematocrit)': 'Sang/Hematocrit',
+	'San-Eritrocits;v.entitic(VCM)': 'Sang/VCM',
+	'San-Eritrocits;vol.entitic(VCM)': 'Sang/VCM',
 	'San-Hemoglobina;c.massa': 'Sang/Hemoglobina',
-	'San-Leucòcits;c.nom.': 'Sang/Leucocits',
-	'San-Limfòcits;c.nom.': 'Sang/Limfocits',
-	'San-Monòcits;c.nom.': 'Sang/Monocits',
-	'San-Neutròfils(segmentats);c.nom.': 'Sang/Neutrofils',
+	'San-Leucocits;c.nom.': 'Sang/Leucocits',
+	'San-Limfocits;c.nom.': 'Sang/Limfocits',
+	'San-Monocits;c.nom.': 'Sang/Monocits',
+	'San-Neutrofils(segmentats);c.nom.': 'Sang/Neutrofils',
 	'San-Plaquetes;c.nom.': 'Sang/Plaquetes',
-	'San-Plaquetes;vol.entític(VPM)': 'Sang/VPM',
-	'Srm-Ac. antimieloperoxi(MPO)(CLIA);c.subst.arb.': 'Serum/Anti-MPO',
-	'Srm-Ac. antiproteinasa 3(PR3)(CLIA);c.subst.arb.': 'Serum/Anti-PR3',
-	'Srm-Ac.(IgG) anti-b2-glicoprot(CLIA);c.subst.arb.': 'Serum/Anti-B2GP(IgG)',
-	'Srm-Ac.(IgG) anticardiolipina(CLIA);c.subst.arb.': 'Serum/Anti-Cardiolipina(IgG)',
-	'Srm-Ac.(IgM) anti-b2-glicoprot(CLIA);c.subst.arb.': 'Serum/Anti-B2GP(IgM)',
-	'Srm-Ac.(IgM) anticardiolipina(CLIA);c.subst.arb.': 'Serum/Anti-Cardiolipina(IgM)',
+	'San-Plaquetes;vol.entitic(VPM)': 'Sang/VPM',
+	'San-Reticulocits;c.nom.': 'Sang/Recitulocits',
+	'Srm-Ac.antimieloperoxi(MPO)(CLIA);c.subst.arb.': 'Serum/Anti-MPO',
+	'Srm-Ac.antiproteinasa 3(PR3)(CLIA);c.subst.arb.': 'Serum/Anti-PR3',
+	'Srm-Ac.(IgG)anti-b2-glicoprot(CLIA);c.subst.arb.': 'Serum/Anti-B2GP(IgG)',
+	'Srm-Ac.(IgG)anticardiolipina(CLIA);c.subst.arb.': 'Serum/Anti-Cardiolipina(IgG)',
+	'Srm-Ac.(IgM)anti-b2-glicoprot(CLIA);c.subst.arb.': 'Serum/Anti-B2GP(IgM)',
+	'Srm-Ac.(IgM)anticardiolipina(CLIA);c.subst.arb.': 'Serum/Anti-Cardiolipina(IgM)',
 	'Srm-Alanina-aminotransferasa;c.cat.': 'Serum/ALT',
-	'Srm-Albúmina;fr.massa': 'Serum/Albumina',
+	'Srm-Albumina;c.massa(CRM 470)': 'Serum/Albumina',
+	'Srm-Albumina;fr.massa': 'Serum/Albumina',
 	'Srm-Anticossos anti-DNA doble cadena;c.subst.arb.': 'Serum/Anti-dsDNA',
-	'Srm-Anticossos antinuclears i citoplasmàtics;c.arb.': 'Serum/ANAs',
+	'Srm-Anticossos antinuclears i citoplasmatics;c.arb.': 'Serum/ANAs',
 	'Srm-Aspartat-aminotransferasa;c.cat.': 'Serum/AST',
+	'Srm-Bilirubina;c.subst.': 'Serum/Bilirrubina',
 	'Srm-Calci(II);c.subst.': 'Serum/Calci',
 	'Srm-Cobalamines;c.subst.': 'Serum/Cobalamines',
-	'Srm-Colesterol(exclòs el d\'HDL);c.subst.': 'Serum/No-HDL',
+	'Srm-Colesterol d\'HDL/Colesterol;quocient subst.': 'Serum/HDL:CT',
 	'Srm-Colesterol d\'HDL;c.subst.': 'Serum/HDL',
-	'Srm-Colesterol d\'LDL;c.subst. (segons Friedewald)': 'Serum/LDL',
-	'Srm-Colesterol d´HDL/Colesterol;quocient subst.': 'Serum/HDL:CT',
+	'Srm-Colesterol d\'LDL;c.subst.(segons Friedewald)': 'Serum/LDL',
+	'Srm-Colesterol(exclos el d\'HDL);c.subst.': 'Serum/No-HDL',
 	'Srm-Colesterol;c.subst.': 'Serum/Colesterol',
 	'Srm-Creatinini;c.subst.': 'Serum/Creatinina',
 	'Srm-Factors reumatoides;c.subst.arb.(OMS 64/2)': 'Serum/FR',
 	'Srm-Folats;c.subst.': 'Serum/Folats',
+	'Srm-Fosfat;c.subst.': 'Serum/Fosfat',
 	'Srm-Fosfatasa alcalina;c.cat.': 'Serum/FA',
-	'Srm-Glucosa;c.subst.(mitjana estimada)': 'Serum/Glucosa',
 	'Srm-Glucosa;c.subst.': 'Serum/Glucosa',
-	'Srm-Ió potassi;c.subst.': 'Serum/Potassi',
-	'Srm-Ió sodi;c.subst.': 'Serum/Sodi',
+	'Srm-Glucosa;c.subst.(mitjana estimada)': 'Serum/Glucosa',
+	'Srm-Io potassi;c.subst.': 'Serum/Potassi',
+	'Srm-Io sodi;c.subst.': 'Serum/Sodi',
+	'Srm-Magnesi(II);c.subst.': 'Serum/Magnesi',
 	'Srm-Proteina C reactiva;c.massa': 'Serum/PCR',
 	'Srm-Proteina;c.massa': 'Serum/Proteina',
 	'Srm-Tirotropina;c.subst.arb.': 'Serum/TSH',
 	'Srm-Tiroxina(no unida a proteina);c.subst.': 'Serum/T4L',
-	'Srm-Triglicèrid;c.subst.': 'Serum/Triglicerid',
+	'Srm-Triglicerid;c.subst.': 'Serum/Triglicerid',
 	'Srm-Urea;c.subst.': 'Serum/Urea',
 	'Srm-alfa 1-Globulina;fr.massa': 'Serum/A1-Globulina',
 	'Srm-alfa 2-Globulina;fr.massa': 'Serum/A2-Globulina',
 	'Srm-beta-Globulina;fr.massa': 'Serum/B-Globulina',
 	'Srm-gamma-Globulina;fr.massa': 'Serum/G-Globulina',
 	'Srm-gamma-Glutamiltransferasa;c.cat.': 'Serum/GGT',
-	'vPla—Excés de base(llocs enllaçants d`H+);c.subst.': 'Sang(v)/EB',
-	'vPla—Hidrogencarbonat;c.subst': 'Sang(v)/HCO3-',
+	'vPla-Exces de base(llocs enllaçants d`H+);c.subst.': 'Sang(v)/EB',
+	'vPla-Hidrogencarbonat;c.subst': 'Sang(v)/Bicarbonat',
 
 }
 
@@ -272,9 +285,18 @@ def normalize_string(s):
 	s = re.sub(r' ([;\(])', r'\1', s)
 	s = re.sub(r'([.;\)]) ', r'\1', s)
 
+	# Fix inconsistent quoting
+	s = re.sub(r'([;\(])[\'"]', r'\1', s)
+	s = re.sub(r'[\'"]([;\)])', r'\1', s)
+
 	# Replace unicode characters with ASCII equivalents
 	s = s.replace('—', '-')
-	s = s.replace('ï', 'i')
+	s = s.replace('´', "'")
+	s = s.replace('à', 'a')
+	s = s.replace('ú', 'u')
+	s = re.sub('[èé]', 'e', s)
+	s = re.sub('[íï]', 'i', s)
+	s = re.sub('[òó]', 'o', s)
 
 	return s.strip()
 
@@ -385,7 +407,7 @@ def try_parse_field_ref_values(content):
 		return OneSidedRefValueInterval(sign=result[1],
 		                                limit=float(result[2]))
 
-	result = re.match(r'(?:Valor|Concentració) desitjable ([<>≤≥]) (\d+(?:\.\d+)?)', content)
+	result = re.match(r'(?:Valor|Concentracio) desitjable ([<>≤≥]) (\d+(?:\.\d+)?)', content)
 	if result is not None:
 		return OneSidedRefValueInterval(sign=result[1],
 		                                limit=float(result[2]))
